/*
 
    hcal version 1.0 - postscript hebrew calendar generator
    copyright 1994,1995 - Steven Weintraub
 
    This program is created and distribed by Steven Weintraub. It
    can be freely distributed without permission as long as this
    head is retained in all files it appears and the LICENCE file
    accompanies it.
 
*/
#include <stdio.h>
#include <setjmp.h>
#include <curses.h>
#include <stdlib.h>
#include <unistd.h>

#ifdef     CURSES /* headers */
#include "curses.h"
#endif /*  CURSES    headers */

#ifdef    XCURSES /* headers */
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#endif /* XCURSES    headers */

#ifndef NO
#define NO  0
#endif
#ifndef YES
#define YES 1
#endif
#ifndef MAX_STR
#define MAX_STR 256
#endif

#define TALK_TOP    1
#define TALK_BOTTOM 2

#define CUR_MENU_INT     1
#define CUR_MENU_STRING  2
#define CUR_MENU_LIST    3
#define CUR_MENU_OLIST   4
#define CUR_MENU_DATA    5
#define CUR_MENU_BUTTON  6
#define CUR_MENU_BOOLEAN 7
#define CUR_MENU_OTHER   8

#define CUR_ALIST_CONTINUE  0
#define CUR_ALIST_END       1
#define CUR_ALIST_REDRAW    2

#ifdef     CURSES /* defines */
#ifndef KEY_DOWN
#define KEY_DOWN 03502
#endif

#ifndef KEY_UP
#define KEY_UP 03503
#endif

#ifndef KEY_LEFT
#define KEY_LEFT 03504
#endif

#ifndef KEY_RIGHT
#define KEY_RIGHT 03505
#endif

#ifndef KEY_PPAGE
#define KEY_PPAGE 03623
#endif

#ifndef KEY_NPAGE
#define KEY_NPAGE 03622
#endif
#endif /*  CURSES    defines */

#ifdef    XCURSES /* defines */
#define CUR_BUTTON_NORETURN -1

#define CUR_WLIST_ROOT                   0
#define CUR_WLIST_MESSAGE                1
#define CUR_WLIST_HOLDER                 2
#define CUR_WLIST_MLISTS                 3

#define CUR_WLIST_BUTTON                 6
#define CUR_WLIST_VERTICAL_SLIDER        7
#define CUR_WLIST_VERTICAL_SLIDER_BOX    8
#define CUR_WLIST_HORIZONTAL_SLIDER      9
#define CUR_WLIST_HORIZONTAL_SLIDER_BOX 10
#define CUR_WLIST_TEXT                  11
#define CUR_WLIST_SCROLLER              12
#define CUR_WLIST_MLIST                 13
#define CUR_WLIST_MLIST_DATA            14

#define CUR_WLIST_OTHER                 20

#define CUR_BUTTON_MARGIN 32
#define CUR_BUTTON_PAD     6
#define CUR_BUTTON_BORDER  4
#define CUR_BUTTON_INNER   4
#define CUR_BUTTON_HEIGHT (CUR_BUTTON_MARGIN + cur_font_height)

#define CUR_SLIDER_WIDTH  20
#define CUR_SLIDER_BORDER  1

#define enlist(cptr,cwindow,ctype,cheight,cwidth) { \
			cptr = (cur_wlist_t *)calloc(sizeof(cur_wlist_t),1); \
			cptr->type = ctype; \
			cptr->process = cur_process_count; \
			cptr->window = cwindow; \
			cptr->height = cheight; \
			cptr->width = cwidth; \
			cptr->warp = NO; \
			cptr->next = cur_win_list; \
			cptr->last = NULL; \
			if (cur_win_list) \
				cur_win_list->last = cptr; \
			cur_win_list = cptr; \
			}

#define delist(item)	{  \
			XDestroyWindow(cur_display,item->window); \
			if (item->last)  \
				item->last->next = item->next;  \
			else  \
				cur_win_list = item->next;  \
			if (item->next)  \
				item->next->last = item->last;  \
			free(item);  \
			}

#define cur_rm_hslider(item)           cur_rm_slider(item)
#define cur_rm_vslider(item)           cur_rm_slider(item)
#define cur_mv_hslider(item1,item2)    cur_mv_slider(item1,item2)
#define cur_mv_vslider(item1,item2)    cur_mv_slider(item1,item2)
#define cur_bump_hslider(item1,item2)  cur_bump_slider(item1,item2)
#define cur_bump_vslider(item1,item2)  cur_bump_slider(item1,item2)
#endif /* XCURSES    defines */

struct cur_str_list {
	char *string;
	struct cur_str_list *last;
	struct cur_str_list *next;
};
typedef struct cur_str_list cur_slist_t;

struct cur_menu_list
	{
	char *name;
	int type;
	int type_special;
	char *s_answer;
	int   s_answer_size;
	int   i_answer;
	int   i_lower;
	int   i_upper;
	cur_slist_t * l_answer;
	union {
		char *value;
		int   i;
	} default_value;
	int have_default;
	int enterable;
	int must_have;
	int edited;
	cur_slist_t *list;
	struct cur_menu_list *next;
	struct cur_menu_list *last;
};
typedef struct cur_menu_list cur_mlist_t;

struct cur_action_list {
	int type;
	void *data;
	struct cur_action_list *next;
	struct cur_action_list *last;
};
typedef struct cur_action_list cur_alist_t;

#ifdef    XCURSES /* structures */
struct cur_window_list
  {
  int type;
  int process;
  Window window;
  int width;
  int height;
  int warp;
  union
    {
    struct 
      {
      char *string;
      int width;
      } message;
    struct 
      {
      char *name;
      int length;
      int width;
      unsigned long return_value;
      char *active_string; /* call routine if letter in this - '*' means all */
      int (*routine)();
      void *parameter1;
      void *parameter2;
      void *parameter3;
      struct cur_window_list *up;
      struct cur_window_list *down;
      struct cur_window_list *alt_next; /* where to go next if last next */
      struct cur_window_list *alt_last; /* where to go next if last last */
      } button;
    struct 
      {
      char **text;
      } text;
    struct 
      {
      struct cur_window_list *child;
      } holder;
    struct 
      {
      int child_width;
      int child_height;
      int child_width_incr;
      int child_height_incr;
      int child_width_base;
      int child_height_base;
      struct cur_window_list *child;
      struct cur_window_list *hslider;
      struct cur_window_list *vslider;
      } scroller;
    struct 
      {
      int vertical;
      int place;
      int size;
      int incr;
      int base;
      struct cur_window_list *parent;
      struct cur_window_list *earlier;
      struct cur_window_list *later;
      } slider;
    struct 
      {
      int incr; /* 1 or -1 */
      struct cur_window_list *parent;
      } slider_box;
    struct 
      {
      struct cur_window_list *first;
      } mlists;
    struct 
      {
      cur_mlist_t *item;
      struct cur_window_list *child;
      struct cur_window_list *next;
      struct cur_window_list *last;
      struct cur_window_list *alt_next; /* where to go next if last next */
      struct cur_window_list *alt_last; /* where to go next if last last */
      } mlist;
    struct 
      {
      cur_mlist_t *item;
      struct cur_window_list *parent;
      int place;
      int first_ch;
      int last_ch;
      } mdata;
    struct 
      {
      int (*routine)();
      void *parameter1;
      void *parameter2;
      void *parameter3;
      struct cur_window_list *parent;
      } other;
    } data;
  struct cur_window_list *next;
  struct cur_window_list *last;
  };

typedef struct cur_window_list cur_wlist_t;
#endif /* XCURSES    structures */

extern int     cur_started;

#ifdef     CURSES /* variables */
extern int     cur_talk_bot_x;
extern int     cur_talk_bot_y;
extern int     cur_talk_height;
extern int     cur_talk_top_x;
extern int     cur_talk_top_y;
extern int     cur_talk_width;
extern WINDOW *cur_talk_window;
extern WINDOW *cur_mlist_window;
#endif /*  CURSES    variables */

#ifdef    DCURSES /* variables */
extern int		 cur_lines;
#endif /* DCURSES    variables */

#ifdef    XCURSES /* variables */
extern unsigned long	 cur_background_color;
extern char		*cur_background_color_name;
extern Window		 cur_base_window;
extern int		 cur_button_height;
extern int		 cur_button_width;
extern cur_wlist_t	*cur_cptr_safe;
extern Display		*cur_display;
extern char		*cur_display_name;
extern unsigned long	 cur_display_height;
extern unsigned long	 cur_display_width;
extern int		 cur_display_number;
extern int		 cur_do_warp;
extern unsigned long	 cur_foreground_color;
extern char		*cur_foreground_color_name;
extern XFontStruct	*cur_font;
extern int		 cur_font_height;
extern int		 cur_font_decent;
extern char		*cur_font_name;
extern GC		 cur_gc;
extern Window		 cur_root_window;
extern cur_wlist_t	 cur_root_wlist;
extern int	 	 cur_routine_value;
extern int		 cur_process_count;
extern int 		 cur_argc;
extern char 		**cur_argv;
extern int		 cur_width_80;
extern cur_wlist_t	*cur_win_list;
#endif /* XCURSES    variables */

#include "cur_modules.h"
