static char version[] =
"hcal version 1.0 - postscript hebrew calendar generator\n\n\
copyright 1994,1995 - Steven Weintraub";

/*

    hcal version 1.0 - postscript hebrew calendar generator
    copyright 1994,1995 - Steven Weintraub
 
    This program is created and distribed by Steven Weintraub. It
    can be freely distributed without permission as long as this
    head is retained in all files it appears and the LICENCE file
    accompanies it.

*/

#include "hcal.h"

#if defined(PROTOTYPES)
int casecmp(char *s1,char *s2) {
#else
int casecmp(s1,s2) char *s1;char *s2; {
#endif
	while (*s1 && *s2 && !(tolower(*s1) - tolower(*s2))) {
		s1++;
		s2++;
	}
	return(*s1 - *s2);
}

#if defined(PROTOTYPES)
int day_of_week(int month,int day,int year) {
#else
int day_of_week(month,day,year) int month;int day;int year; {
#endif
  int year_diffs;
  int total_days;

  year_diffs = year - BASE_YEAR;
  total_days = 1;                        /* Jan 1, 1760 is a Tuesday - 1 */
  total_days += (365 * year_diffs);      /* + 365 * years */
  total_days += (year_diffs / 4) + 1;    /* + leap years */
  total_days -= (year / 100) - 17;       /* - non-leap years at cent (-17c) */
  total_days += (year / 400) - 4;        /* + leap years on 4 cent (-16/4c) */
  total_days += month_lengths[month];    /* month */
  total_days += day;                     /* day */
  if (month < 2 && year % 4 == 0 && (year % 100 != 0 || year % 400 == 0)) 
    total_days--;                        /* leap year correction */
  return(total_days % 7);
  }


#if defined(PROTOTYPES)
void report_error(char *error) {
#else
void report_error(error) char *error; {
#endif
#ifndef BATCH
	cur_error(error);
#else
	fprintf(stderr,"%s\n",error);
#endif
}

#if defined(PROTOTYPES)
void display_it(char **it) {
#else
void display_it(it) char **it; {
#endif
	int	  count;
	int	  ch;
	char	**iptr;

#ifndef BATCH
	if (cur_started)
		cur_text_dsp(it);
	else {
#endif
		for (iptr = it,count = 0;*iptr;iptr++,count++) {
			fprintf(stderr,"%s\n",*iptr);
			if (count > 23) {
				count = 0;
				fprintf(stderr,"\nHit <cr> for more");
				while (ch = getchar(),ch != EOF && ch != '\n')
					;
				fputc('\n',stderr);
			}
		}
		fprintf(stderr,"\nHit <cr> to continue");
		while (ch = getchar(),ch != EOF && ch != '\n')
			;
		fputc('\n',stderr);
#ifndef BATCH
	}
#endif
}

#ifndef BATCH
#if defined(PROTOTYPES)
void finalize(void) {
#else
void finalize() {
#endif
	cur_stop();
}
#endif

#ifndef BATCH
#if defined(PROTOTYPES)
FILE *get_file(char *type,char *question,char *name) {
#else
FILE *get_file(type,question,name) char *type;char *question;char *name; {
#endif
	struct stat	_buffer;
	struct stat	*buffer = &_buffer;
	FILE		*file;

	cur_strd(question,name,name);
	if (stat(name,buffer) == -1) {
		if (errno != ENOENT) {
			cur_msg_win("That file is not a proper filename");
			return NULL;
		}
	} else if (*type == 'w') {
		if (access(name,W_OK) == 0) {
			if (!cur_ask_y_n(
	"File exists, Do you wish you overwrite it","Overwrite"))
				return NULL;
		} else {
			cur_msg_win(
	"You do not have permission to write to that file");
			return NULL;
		}
	} else {
		if (access(name,R_OK) != 0) {
			cur_msg_win(
	"You do not have permission to read that file");
			return NULL;
		}
	}
	if ((file = fopen(name,type)) == NULL)
		cur_msg_win("Could not open the file");
	return file;
}
#endif

#if defined(PROTOTYPES)
int julian_date(int month,int day,int year) {
#else
int julian_date(month,day,year) int month;int day;int year; {
#endif
	int	jday;

	jday = month_lengths[month] + day;
	if (month > 1 && (year % 4) == 0 && year != 2000)
		jday++;
	return(jday);
}


#ifndef BATCH
#if defined(PROTOTYPES)
cur_mlist_t *mk_menu(char **menu) {
#else
cur_mlist_t *mk_menu(menu) char **menu; {
#endif
	cur_mlist_t	 *mlist;
	char		**mptr;

	mlist = NULL;
	for (mptr = menu;*mptr;mptr++)
		mlist = cur_mlist_add(mlist,cur_menu_parse(*mptr));
	return mlist;
}
#endif

#if defined(PROTOTYPES)
char *mk_string(char *string) {
#else
char *mk_string(string) char *string; {
#endif
	char	*rptr;

	if (!string) {
		rptr = (char *)new(1);
		*rptr = '\0';
	} else {
		rptr = (char *)new(strlen(string) + 1);
		strcpy(rptr,string);
	}
	return rptr;
}

#if defined(PROTOTYPES)
void month_day(int yday,int year,int *day,int *month) {
#else
void month_day(yday,year,day,month) int yday;int year;int *day;int *month; {
#endif
	int	loop;

	if ((year % 4) == 0 && year != 2000 && year != 100 && yday > 60)
		yday--;
	for (loop = 0;loop < 11 && month_lengths[loop + 1] < yday;loop++)
		;
	*month = loop;
	*day = yday - month_lengths[*month];
}

#if defined(PROTOTYPES)
void *new(int length) {
#else
void *new(length) int length; {
#endif
	void	*retptr;

	if ((retptr = (void *)calloc(length,1)) == NULL) {
#ifndef BATCH
		finalize();
#endif
		fprintf(stderr,"Can not allocate %d bytes of memory\n");
		exit(1);
	}
	return(retptr);
}

static float bsize[] = {
	 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
	/*						  */
	 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
	/*						  */
	 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
	/*						  */
	 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
	/*						  */
	 2.78, 3.33, 4.74, 5.56, 5.56, 8.89, 7.23, 2.78,
	/* ' '   !      "     #     $     %     &     '  */
	 3.33, 3.33, 3.89, 5.84, 2.78, 3.33, 2.78, 2.78,
	/* (      )     *     +     ,     -     .     /  */
	 5.56, 5.56, 5.56, 5.56, 5.56, 5.56, 5.56, 5.56,
	/* 0      1     2     3     4     5     6     7  */
	 5.56, 5.56, 3.33, 3.33, 5.84, 5.84, 5.84, 6.11,
	/* 8      9     :     ;     <     =     >     ?  */
	 9.75, 7.22, 7.22, 7.22, 7.22, 6.67, 6.11, 7.78,
	/* @      A     B     C     D     E     F     G  */
	 7.22, 2.78, 5.56, 7.22, 6.11, 8.33, 7.22, 7.78,
	/* H      I     J     K     L     M     N     O  */
	 6.67, 7.78, 7.22, 6.67, 6.11, 7.22, 6.67, 9.44,
	/* P      Q     R     S     T     U     V     W  */
	 6.67, 6.67, 6.11, 3.33, 2.78, 3.33, 5.84, 5.56,
	/* X      Y     Z     [     \     ]     ^     _  */
	 2.78, 5.56, 6.11, 5.56, 6.11, 5.56, 3.33, 6.11,
	/* 	      a     b     c     d     e     f     g  */
	 6.11, 2.78, 2.78, 5.56, 2.78, 8.89, 6.11, 6.11,
	/* h      i     j     k     l     m     n     o  */
	 6.11, 6.11, 3.89, 5.56, 3.33, 6.11, 5.56, 7.78,
	/* p      q     r     s     t     u     v     w  */
	 5.56, 5.56, 5.00, 3.89, 2.80, 3.89, 5.84, 0.00
	/* x      y     z     {     |     }     ~	  */
 };

static float fsize[] = {
	 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
	/*						  */
	 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
	/*						  */
	 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
	/*						  */
	 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
	/*						  */
	 2.78, 2.78, 3.55, 5.56, 5.56, 8.89, 6.67, 2.22,
	/*        !     "     #     $     %     &     '  */
	 3.33, 3.33, 3.89, 5.84, 2.78, 3.33, 2.78, 2.78,
	/* (      )     *     +     ,     -     .     /  */
	 5.56, 5.56, 5.56, 5.56, 5.56, 5.56, 5.56, 5.56,
	/* 0      1     2     3     4     5     6     7  */
	 5.56, 5.56, 2.78, 2.78, 5.84, 5.84, 5.84, 5.56,
	/* 8      9     :     ;     <     =     >     ?  */
	10.15, 6.67, 6.67, 7.22, 7.22, 6.67, 6.11, 7.78,
	/* @      A     B     C     D     E     F     G  */
	 7.22, 2.78, 5.00, 6.67, 5.56, 8.33, 7.22, 7.78,
	/* H      I     J     K     L     M     N     O  */
	 6.67, 7.78, 7.22, 6.67, 6.11, 7.22, 6.67, 9.44,
	/* P      Q     R     S     T     U     V     W  */
	 6.67, 6.67, 6.11, 2.78, 2.78, 2.78, 4.69, 5.56,
	/* X      Y     Z     [     \     ]     ^     _  */
	 2.22, 5.56, 5.56, 5.00, 5.56, 5.56, 2.78, 5.56,
	/* 	      a     b     c     d     e     f     g  */
	 5.56, 2.22, 2.22, 5.00, 2.22, 8.33, 5.56, 5.56,
	/* h      i     j     k     l     m     n     o  */
	 5.56, 5.56, 3.33, 5.00, 2.78, 5.56, 5.00, 7.22,
	/* p      q     r     s     t     u     v     w  */
	 5.00, 5.00, 5.00, 3.34, 2.60, 3.34, 5.84, 0.00
	/* x      y     z     {     |     }     ~	  */
 };

#if defined(PROTOTYPES)
float pstrlen(char *string,int is_bold) {
#else
float pstrlen(string,is_bold) char *string;int is_bold; {
#endif
	float	 sum	= 0;
	float   *table;
	char	*s;

	if (is_bold)
		table = bsize;
	else
		table = fsize;

	for (s = string;*s;s++)
		sum +=  *(table + *s);
	return(sum);
}

#ifndef BATCH
#if defined(PROTOTYPES)
int size_verify(cur_mlist_t *list) {
#else
int size_verify(list) cur_mlist_t *list; {
#endif
	char		*sptr;
	time_info_t	 time;

	time.word1 = list->next->next->i_answer;
	time.word2 = list->next->next->i_lower;
	sptr = items_fit(&time,list->next->next->next->s_answer,
			list->next->i_answer,
			list->i_answer);
	if (sptr) {
		cur_msg_win(sptr);
		return 0;
	}
	return 1;
}
#endif

#if defined(PROTOTYPES)
char *stradd(char *first,char *separator,char *last) {
#else
char *stradd(first,separator,last) char *first;char *separator;char *last; {
#endif
	int	 length;
	char	*retval;

	if (first)
		length = strlen(first);
	else
		length = 0;
	if (first && last && separator)
		length += strlen(separator);
	if (last)
		length += strlen(last);

	if (length)
		retval = (char *)new(length + 1);
	else
		return NULL;

	if (first)
		strcpy(retval,first);
	else
		*retval = '\0';
	if (first && last && separator)
		strcat(retval,separator);
	if (last)
		strcat(retval,last);

	free(first);

	return retval;
}
