static char version[] =
"hcal version 1.0 - postscript hebrew calendar generator\n\n\
copyright 1994,1995 - Steven Weintraub";

/*
 
    hcal version 1.0 - postscript hebrew calendar generator
    copyright 1994,1995 - Steven Weintraub
 
    This program is created and distribed by Steven Weintraub. It
    can be freely distributed without permission as long as this
    head is retained in all files it appears and the LICENCE file
    accompanies it.
 
*/

#include "hcal.h"

#if defined(PROTOTYPES)
int mprc_delete(int type,void *data) {
#else
int mprc_delete(type,data) int type;void *data; {
#endif
	int		 date;
	info_t		*info;
	day_data_t	*tday;

	date = type & TYPE_BACK;
	tday = day + date;
	type = type - date;
	switch (type) {
		case TYPE_DELETE:
		case TYPE_SHOW_ALL:
		case TYPE_SHOW_SOME:
		case TYPE_ADD:
		case TYPE_ADD_MISC:
			cur_msg_win("Can not Delete a menu option");
			changed = NO;
			break;
		case TYPE_QUIT:
			changed = NO;
			asking = NO;
			break;
		case TYPE_CHANUKAH :
			tday->chanukah = 0;
			changed_all = YES;
			break;
		case TYPE_OMER :
			tday->omer = 0;
			changed_all = YES;
			break;
		case TYPE_CANDLES :
			tday->candles.word1 = 0;
			tday->candles.word2 = 0;
			changed_all = YES;
			break;
		case TYPE_JDATE :
			if (tday->jmonth)
				free(tday->jmonth);
			tday->jmonth = NULL;
			tday->jday = 0;
			tday->jyear = 0;
			changed_all = YES;
			break;
		case TYPE_SHAHARIT :
			tday->shaharit.word1 = 0;
			tday->shaharit.word2 = 0;
			changed_all = YES;
			break;
		case TYPE_MINCHA :
			tday->mincha.word1 = 0;
			tday->mincha.word2 = 0;
			changed_all = YES;
			break;
		case TYPE_MARIV :
			tday->mariv.word1 = 0;
			tday->mariv.word2 = 0;
			changed_all = YES;
			break;
		case TYPE_HOLIDAY :
			tday->holiday = NO;
			changed_all = YES;
			break;
		case TYPE_MITZVAH :
			if (tday->mitzvah->who)
				free(tday->mitzvah->who);
			if (tday->mitzvah->schedule)
				free(tday->mitzvah->schedule);
			if (tday->mitzvah)
				free(tday->mitzvah);
			tday->mitzvah = NULL;
			changed = YES;
			break;
		case TYPE_SHABBAT :
			if (tday->shabbat) {
				if (tday->shabbat->name)
					free(tday->shabbat->name);
				if (tday->shabbat->schedule)
					free(tday->shabbat->schedule);
				if (tday->shabbat->torah)
					free(tday->shabbat->torah);
				if (tday->shabbat->prophet)
					free(tday->shabbat->prophet);
				free(tday->shabbat);
			}
			tday->shabbat = NULL;
			changed = YES;
			break;
		case TYPE_INFO :
			info = (info_t *)data;
			if (info->next)
				info->next->last = info->last;
			else
				tday->info_last = info->last;
			if (info->last)
				info->last->next = info->next;
			else
				tday->info = info->next;
			if (info->calendar)
				free(info->calendar);
			if (info->schedule)
				free(info->schedule);
			if (info)
				free(info);
			changed = YES;
	}
	delete = NO;
	if (changed || changed_all)
		dirty = YES;
	return (changed || changed_all) ? CUR_ALIST_END : CUR_ALIST_CONTINUE;
}

#if defined(PROTOTYPES)
int mprc_edit(int action,int type,void *data) {
#else
int mprc_edit(action,type,data) int action;int type;void *data; {
#endif
	int	date;
	int	edits;
	int	save1,save2,save3;
	time_info_t tsave1,tsave2,tsave3;

	edits = CUR_ALIST_CONTINUE;
	if (delete || action == 'd') {
		if (!type && !data)
			type = TYPE_DELETE;
		else
			return mprc_delete(type,data);
	}
	date = type & TYPE_BACK;
	type = type - date;
	if (action == 'a')
		type = TYPE_ADD;
	switch (type) {
		case TYPE_DELETE:
			cur_msg_win("Clear This Dialog then Select Item to Delete");
			delete = YES;
			break;
		case TYPE_SHOW_ALL:
			edits = CUR_ALIST_END;
			show_all = YES;
			changed = YES;
			break;
		case TYPE_SHOW_SOME:
			show_all = NO;
			edits = CUR_ALIST_END;
			changed = YES;
			break;
		case TYPE_QUIT:
			asking = NO;
			edits = CUR_ALIST_END;
			changed = YES;
			break;
		case TYPE_ADD:
			if (add_item()) {
				changed = YES;
				edits = CUR_ALIST_END;
			}
			break;
		case TYPE_ADD_MISC:
			if (add_misc()) {
				changed = YES;
				edits = CUR_ALIST_END;
			}
			break;
		case TYPE_INFO :
			if (edit_item(date,(info_t *)data)) {
				changed = YES;
				edits = CUR_ALIST_REDRAW;
			}
			break;
		case TYPE_SHAHARIT :
		case TYPE_MINCHA :
		case TYPE_MARIV :
			tsave1.word1 = day[date].shaharit.word1;
			tsave1.word2 = day[date].shaharit.word2;
			tsave2.word1 = day[date].mincha.word1;
			tsave2.word2 = day[date].mincha.word2;
			tsave3.word1 = day[date].mariv.word1;
			tsave3.word2 = day[date].mariv.word2;
			if (mprc_minyan(' ',TYPE_MINYAND | date,NULL) ==
					CUR_ALIST_REDRAW) {
				edits = CUR_ALIST_REDRAW;
			if (tsave1.word1 != day[date].shaharit.word1 ||
			    tsave1.word2 != day[date].shaharit.word2 ||
			    tsave2.word1 != day[date].mincha.word1 ||
			    tsave2.word2 != day[date].mincha.word2 ||
			    tsave3.word1 != day[date].mariv.word1 ||
			    tsave3.word2 != day[date].mariv.word2)
				edits = CUR_ALIST_END;
			}
			break;
		case TYPE_EDIT_MINYAN:
			edit_minyan();
			changed_all = YES;
			edits = CUR_ALIST_END;
			break;
		case TYPE_ADD_MITZVAH:
			if (add_mitzvah()) {
				changed = YES;
				edits = CUR_ALIST_END;
			}
			break;
		case TYPE_MITZVAH :
			if (edit_mitzvah(date,(day_data_t *)data)) {
				changed = YES;
				edits = CUR_ALIST_REDRAW;
			}
			break;
		case TYPE_SHABBAT :
			if (edit_shabbat(date,(day_data_t *)data)) {
				changed = YES;
				edits = CUR_ALIST_REDRAW;
			}
			break;
		case TYPE_CHANUKAH :
			save1 = ((day_data_t *)data)->chanukah;
			((day_data_t *)data)->chanukah = cur_ask_num_lud(
	"Day of Chanukah icon to put on calendar (0 = none)","Day of Chanukah",
				0,8,((day_data_t *)data)->chanukah);
			if (save1 != ((day_data_t *)data)->chanukah) {
				changed_all = YES;
				if (!save1 || !((day_data_t *)data)->chanukah)
					edits = CUR_ALIST_END;
				else
					edits = CUR_ALIST_REDRAW;
			}
			break;
		case TYPE_OMER :
			save1 = ((day_data_t *)data)->omer;
			((day_data_t *)data)->omer = cur_ask_num_lud(
	"Day of omer icon to put on calendar (0 = none)","Day of omer",0,49,
				((day_data_t *)data)->omer);
			if (save1 != ((day_data_t *)data)->omer) {
				changed_all = YES;
				if (!save1 || !((day_data_t *)data)->omer)
					edits = CUR_ALIST_END;
				else
					edits = CUR_ALIST_REDRAW;
			}
			break;
		case TYPE_HOLIDAY :
			cur_msg_win(
				"This is not a an editable item - use add/delete");
			break;
		case TYPE_CANDLES :
			save1 = day[date].candles.word1;
			modify_candles(date);
			if (save1 != day[date].candles.word1) {
				if (!save1 == 0 || day[date].candles.word1)
					edits = CUR_ALIST_END;
				else
					edits = CUR_ALIST_REDRAW;
			}
			break;
		case TYPE_JDATE:
			if (edit_jdate(date,(day_data_t *)data)) {
				changed_all = YES;
				edits = CUR_ALIST_REDRAW;
			}
			break;
	}
	return edits;
}

#if defined(PROTOTYPES)
int mprc_main(int action,int type,void *data) {
#else
int mprc_main(action,type,data) int action;int type;void *data; {
#endif
	int retval;

	retval = CUR_ALIST_CONTINUE;
	switch (type) {
		case TYPE_MAIN_MONTH : cmd_month(); break;
		case TYPE_MAIN_EDIT  : cmd_edit();  break;
		case TYPE_MAIN_DATA  : cmd_data();  break;
		case TYPE_MAIN_SAVE  : cmd_save();  break;
		case TYPE_MAIN_READ  : cmd_read();  break;
		case TYPE_MAIN_PRINT : cmd_print(); break;
		case TYPE_MAIN_QUIT  : 
			cmd_quit();
			if (!dirty)
				retval = CUR_ALIST_END;
			break;
	}
	return retval;
}

#if defined(PROTOTYPES)
int mprc_minyan(int action,int type,void *data) {
#else
int mprc_minyan(action,type,data) int action;int type;void *data; {
#endif
	int		 loop;
	cur_mlist_t	*menu;
	cur_mlist_t	*mptr;
	int		 this_day;
	int		 weekday;
	int		 edits;
	time_info_t	 time1;
	time_info_t	 time2;
	time_info_t	 time3;

	edits = NO;
	this_day = type & TYPE_BACK;
	weekday = DAY_OF_WEEK(this_day);
	type = type - this_day;
	switch (type) {
		case TYPE_MINYANW:
			if (!menu_minyan_edit)
				menu_minyan_edit = mk_menu(
						menu_list_minyan_edit);
			cur_menu_reset(menu_minyan_edit);

			mptr = menu_minyan_edit;
			if (this_day < ROSH)
				sprintf(mptr->s_answer,"%s",weekdays[this_day]);
			else if (this_day == ROSH)
				strcpy(mptr->s_answer,"Rosh Chodesh");
			else if (this_day == CHAG)
				strcpy(mptr->s_answer,"Chag");
			mptr = mptr->next;
			mlist_set_time(0,mptr,&(minyan[this_day][SHAHARIT]));
			mptr = mptr->next;
			mlist_set_time(0,mptr,&(minyan[this_day][MINCHA]));
			mptr = mptr->next;
			mlist_set_time(0,mptr,&(minyan[this_day][MARIV]));

			if (cur_menu_ask("Edit the Minyan Time",
					menu_minyan_edit,NULL)) {
				dirty = YES;
				tdirty = YES;
				changed_all = YES;
				edits = YES;
				mptr = menu_minyan_edit->next;
				time1.word1 = mptr->i_answer;
				time1.word2 = mptr->i_lower;
				time2.word1 = mptr->next->i_answer;
				time2.word2 = mptr->next->i_lower;
				time3.word1 = mptr->next->next->i_answer;
				time3.word2 = mptr->next->next->i_lower;
				if (this_day < ROSH)
					do_minyan_week(this_day,&time1,
							&time2,&time3);
				else if (this_day == ROSH)
					do_minyan_rosh(&time1,&time2,&time3);
				else
					do_minyan_chag(&time1,&time2,&time3);
			}
			break;
		case TYPE_MINYAND:
			if (!menu_minyan_edit)
				menu_minyan_edit = mk_menu(
						menu_list_minyan_edit);
			cur_menu_reset(menu_minyan_edit);

			mptr = menu_minyan_edit;
			sprintf(mptr->s_answer,"%s %d",month[current_month],
					this_day);
			mptr = mptr->next;
			mlist_set_time(this_day,mptr,&day[this_day].shaharit);
			mptr = mptr->next;
			mlist_set_time(this_day,mptr,&day[this_day].mincha);
			mptr = mptr->next;
			mlist_set_time(this_day,mptr,&day[this_day].mariv);

			if (cur_menu_ask("Edit the Minyan Time",
					menu_minyan_edit,NULL)) {
				dirty = YES;
				tdirty = YES;
				changed_all = YES;
				edits = YES;
				mptr = menu_minyan_edit->next;
				time1.word1 = mptr->i_answer;
				time1.word2 = mptr->i_lower;
				time2.word1 = mptr->next->i_answer;
				time2.word2 = mptr->next->i_lower;
				time3.word1 = mptr->next->next->i_answer;
				time3.word2 = mptr->next->next->i_lower;
				do_minyan_day(this_day,&time1,&time2,&time3);
			}
			break;

		case TYPE_QUIT:
			asking = NO;
			break;
	}
	return edits ? CUR_ALIST_REDRAW : 
		(asking ? CUR_ALIST_CONTINUE : CUR_ALIST_END);
}

#if defined(PROTOTYPES)
int mprc_misc(int action,int type,void *data) {
#else
int mprc_misc(action,type,data) int action;int type;void *data; {
#endif
	int	date;

	type = type & TYPE_MASK;
	switch (type) {
		case TYPE_SHABBAT :
			changed = add_shabbat();
			break;
		case TYPE_JDATE :
		case TYPE_HOLIDAY :
		case TYPE_CANDLES :
		case TYPE_CHANUKAH :
		case TYPE_OMER :
			date = cur_ask_num_lud(
			       "What day do you what to add this to (0 = none)",
				"What day",0,numdays,0);
			if (date) {
				switch (type) {
					case TYPE_JDATE :
						if (day[date].jmonth)
							cur_msg_win(
		"This day already has Jewish date information");
						else
							changed_all =
								add_jdate(date);
						break;
					case TYPE_HOLIDAY :
						day[date].holiday = YES;
						changed_all = YES;
						break;
					case TYPE_CANDLES :
						modify_candles(date);
						break;
					case TYPE_CHANUKAH :
						day[date].chanukah =
							cur_ask_num_lud(
"What day of Chanukah icon do you want","What day",0,8,day[date].chanukah);
						changed_all = YES;
						break;
					case TYPE_OMER :
						day[date].omer =
							cur_ask_num_lud(
"What day of omer icon do you want","What day",0,49,day[date].omer);
						changed_all = YES;
						break;
					}
				}
			break;
		default:
			break;
	}
	return CUR_ALIST_END;
}

#if defined(PROTOTYPES)
char *mprt_edit(int type,void *data,char *string) {
#else
char *mprt_edit(type,data,string)
int type;void *data;char *string; {
#endif
	int       date;
	day_data_t *tday;
	info_t     *info;

	date = type & TYPE_BACK;
	tday = (day_data_t *)data;
	info = (info_t *)data;

	switch (type & TYPE_MASK) {
		case TYPE_ADD:
			strcpy(string,"Add An Item");
			break;
		case TYPE_ADD_MISC:
			strcpy(string,"Add Some Miscellaneous Information");
			break;
		case TYPE_ADD_MITZVAH:
			strcpy(string,"Add A Bar/Bat Mitzvah");
			break;
		case TYPE_EDIT_MINYAN:
			strcpy(string,"Edit Minyan Information");
			break;
		case TYPE_DELETE:
			strcpy(string,"Delete An Entry");
			break;
		case TYPE_SHOW_SOME:
			strcpy(string,"Show Only Major Entries");
			break;
		case TYPE_SHOW_ALL:
			strcpy(string,"Show All Entries");
			break;
		case TYPE_QUIT:
			strcpy(string,"Quit This Menu");
			break;
		case TYPE_JDATE :
			sprintf(string,"%3.3s %d - %s %d",
			    month[current_month],date,tday->jmonth,tday->jday);
			break;
		case TYPE_SHAHARIT :
			sprintf(string,"%3.3s %d - Shaharit %s",
			    month[current_month],date,
			    format_time(date,&tday->shaharit,TIME_FORMAT_FULL));
			break;
		case TYPE_MINCHA :
			sprintf(string,"%3.3s %d - Mincha   %s",
			    month[current_month],date,
			    format_time(date,&tday->mincha,TIME_FORMAT_FULL));
			break;
		case TYPE_MARIV :
			sprintf(string,"%3.3s %d - Mariv    %s",
			    month[current_month],date,
			    format_time(date,&tday->mariv,TIME_FORMAT_FULL));
			break;
		case TYPE_MITZVAH :
			if (tday->mitzvah->place == 1)
				sprintf(string,"%3.3s %d - %s Mitzvah of %s",
					month[current_month],date,
					mitztypes[tday->mitzvah->type],
					tday->mitzvah->who);
			else
				sprintf(string,"%3.3s %d - %s %s Mitzvah",
					month[current_month],date,
					tday->mitzvah->who,
					mitztypes[tday->mitzvah->type]);
			break;
		case TYPE_HOLIDAY :
			sprintf(string,"%3.3s %d - Is A Holiday",
				month[current_month],date);
			break;
		case TYPE_CANDLES :
			sprintf(string,"%3.3s %d - Light %s at %s",
				month[current_month],date,
				tday->candles.word1 == TIME_CANDLES_HAVDALAH ?
				"Havdalah":"Candles",
				format_time(date,&tday->candles,
						TIME_FORMAT_FULL));
			break;
		case TYPE_SHABBAT :
			sprintf(string,"%3.3s %d - Shabbat %s (%s %s)",
				month[current_month],date,
				tday->shabbat->name,tday->shabbat->torah,
				tday->shabbat->prophet);
			break;
		case TYPE_CHANUKAH :
			sprintf(string,"%3.3s %d - %d Day of Chanukah icon",
				month[current_month],date,tday->chanukah);
			break;
		case TYPE_OMER :
			sprintf(string,
				"%3.3s %d - %d Day of Counting Omer icon",
			    month[current_month],date,tday->omer);
			break;
		case TYPE_INFO :
			if (*info->calendar) {
				if (info->time.word1)
					sprintf(string,"%3.3s %d - %s %-30.30s",
						month[current_month],date,
						format_time(date,&info->time,TIME_FORMAT_FULL),
						info->calendar);
				else
					sprintf(string,"%3.3s %d - %-30.30s",
						month[current_month],date,
						info->calendar);
			}
			else if (info->time.word1)
				sprintf(string,
					"%3.3s %d - (on back) %s %-30.30s",
					month[current_month],date,
					format_time(date,&info->time,TIME_FORMAT_FULL),
					info->schedule);
			else
				sprintf(string,"%3.3s %d - (on back) %-30.30s",
				month[current_month],date,info->schedule);
			break;
		default:
	sprintf(string,"Unknown entry 0x%x",type);
			break;
	}
	string[75] = '\0';
	return(string);
}

#if defined(PROTOTYPES)
char *mprt_main(int type,void *data,char *string) {
#else
char *mprt_main(type,data,string)
int type;void *data;char *string; {
#endif
	switch(type) {
		case TYPE_MAIN_MONTH :
			strcpy(string,"Set the month calendar is for"); 
			break;
		case TYPE_MAIN_EDIT :
			strcpy(string,
			    "Add/Show/Edit/Delete information on the calendar");
			break;
		case TYPE_MAIN_DATA :
			strcpy(string,"Set location and time data");
			break;
		case TYPE_MAIN_SAVE :
			strcpy(string,"Save the current data to a file");
			break;
		case TYPE_MAIN_READ :
			strcpy(string,"Read in a data file ");
			break;
		case TYPE_MAIN_PRINT :
			strcpy(string,"Output the calendar and schedule file");
			break;
		case TYPE_MAIN_QUIT :
			strcpy(string,"Quit");
			break;
	}
	return(string);
}

#if defined(PROTOTYPES)
char *mprt_minyan(int type,void *data,char *string) {
#else
char *mprt_minyan(type,data,string)
int type;void *data;char *string; {
#endif
	char	 mariv[MAX_STR];
	char	 mincha[MAX_STR];
	char	 shaharit[MAX_STR];
	int      day = type & TYPE_BACK;

	switch (type & TYPE_MASK) {
		case TYPE_MINYANW:
			strcpy(shaharit,format_time(0,&(((time_info_t *)data)[SHAHARIT]),TIME_FORMAT_ACTUAL));
			strcpy(mincha,format_time(0,&(((time_info_t *)data)[MINCHA]),TIME_FORMAT_ACTUAL));
			strcpy(mariv,format_time(0,&(((time_info_t *)data)[MARIV]),TIME_FORMAT_ACTUAL));
			sprintf(string,"%-14s      %s   %s   %s",
					(day < ROSH) ? weekdays[day] :
					    (day == ROSH) ? "Rosh Chodesh" : "Chag",
					shaharit,mincha,mariv);
			break;
		case TYPE_MINYAND:
			strcpy(shaharit,format_time(0,&(((day_data_t *)data)->shaharit),TIME_FORMAT_ACTUAL));
			strcpy(mincha,format_time(0,&(((day_data_t *)data)->mincha),TIME_FORMAT_ACTUAL));
			strcpy(mariv,format_time(0,&(((day_data_t *)data)->mariv),TIME_FORMAT_ACTUAL));
			sprintf(string,"%-14s %2d   %s   %s   %s",
					month[current_month],day,
					shaharit,mincha,mariv);
			break;
		case TYPE_QUIT:
			strcpy(string,"Quit This Menu");
			break;
	}
	string[75] = '\0';
	return(string);
}

#if defined(PROTOTYPES)
char *mprt_misc(int type,void *data,char *string) {
#else
char *mprt_misc(type,data,string)
int type;void *data;char *string; {
#endif
	int		 date;
	day_data_t	*tday;
	info_t		*info;

	date = type & TYPE_BACK;
	tday = (day_data_t *)data;
	info = (info_t *)data;

	switch (type & TYPE_MASK) {
		case TYPE_QUIT:
			strcpy(string,"Don't Add Anything");
			break;
		case TYPE_JDATE :
			strcpy(string,"Add Jewish Date Information");
			break;
		case TYPE_HOLIDAY :
			strcpy(string,"Mark Day as Holiday");
			break;
		case TYPE_CANDLES :
			strcpy(string,
				"Add Candle/Havdalah lighting information");
			break;
		case TYPE_SHABBAT :
			strcpy(string,"Add a Shabbat to the calendar");
			break;
		case TYPE_CHANUKAH :
			strcpy(string,"Add a Chanukiah icon to the calendar");
			break;
		case TYPE_OMER :
			strcpy(string,"Add an Omer icon to the calendar");
			break;
		default:
			sprintf(string,"Unknown entry 0x%x",type);
			break;
	}
	return(string);
}
