static char version[] =
"hcal version 1.0 - postscript hebrew calendar generator\n\n\
copyright 1994,1995 - Steven Weintraub";

/*
 
    hcal version 1.0 - postscript hebrew calendar generator
    copyright 1994,1995 - Steven Weintraub
 
    This program is created and distribed by Steven Weintraub. It
    can be freely distributed without permission as long as this
    head is retained in all files it appears and the LICENCE file
    accompanies it.
 
*/

#include "hcal.h"

#if defined(PROTOTYPES)
void print_city(FILE *file,city_t *city) {
#else
void print_city(file,city) FILE *file;city_t *city; {
#endif
	fprintf(file,"City %s\n",city->name);
	if (city->latdeg >= 0 && city->latmin >= 0)
		fprintf(file,"Latitude %d:%2.2d:%2.2d N\n",
				city->latdeg,city->latmin,city->latsec);
	else
		fprintf(file,"Latitude %d:%2.2d:%2.2d S\n",
				-city->latdeg,-city->latmin,-city->latsec);
	if (city->longdeg >= 0 && city->longmin >= 0)
		fprintf(file,"Longitude %d:%2.2d:%2.2d W\n",
				city->longdeg,city->longmin,city->longsec);
	else
		fprintf(file,"Longitude %d:%2.2d:%2.2d E\n",
				-city->longdeg,-city->longmin,-city->longsec);

	if (city->dst_start_month > 0) {
		fprintf(file,"DST starts %s week in %s\n",
				weekth[city->dst_start_day >= 0 ?
				city->dst_start_day: 4 - city->dst_start_day],
				month[city->dst_start_month]);
		fprintf(file,"DST ends %s week in %s\n",
				weekth[city->dst_end_day >= 0 ?
				city->dst_end_day: 4 - city->dst_end_day],
				month[city->dst_end_month]);
	}
	fprintf(file,"Timezone %d hours off GMT\n\n",city->tzone);
}

#if defined(PROTOTYPES)
void print_minyan(FILE *file,int do_month) {
#else
void print_minyan(file,do_month) FILE *file;int do_month; {
#endif
	int dow;
	int first;
	int loop;
	char shaharit[MAX_STR];
	char mincha[MAX_STR];
	char mariv[MAX_STR];

	first = YES;
	for (loop = 0;loop < 9;loop++) {
		if (minyan[loop][0].word1|| minyan[loop][1].word1 || 
				minyan[loop][2].word1) {
			strcpy(shaharit,format_time(0,&(minyan[loop][SHAHARIT]),
					TIME_FORMAT_ACTUAL));
			strcpy(mincha,format_time(0,&(minyan[loop][MINCHA]),
					TIME_FORMAT_ACTUAL));
			strcpy(mariv,format_time(0,&(minyan[loop][MARIV]),
					TIME_FORMAT_ACTUAL));
			fprintf(file,"%-10s %-10s   %s   	%s   	%s\n",
					first ? "Minyan" : "",
					(loop < ROSH) ? weekdays[loop] : 
					    (loop == ROSH) ? "Rosh Chodesh" :"Chag",
					shaharit,mincha,mariv);
			first = NO;
		}
	}
	if (do_month) {
		for (loop = 1;loop <= numdays;loop++) {
			dow = DAY_OF_WEEK(loop);
			if (minyan[dow][SHAHARIT].word1 != 
						day[loop].shaharit.word1 ||
				minyan[dow][SHAHARIT].word2 != 
						day[loop].shaharit.word2 ||
				minyan[dow][MINCHA].word1 != 
						day[loop].mincha.word1 ||
				minyan[dow][MINCHA].word2 != 
						day[loop].mincha.word2 ||
				minyan[dow][MARIV].word1 != 
						day[loop].mariv.word1 ||
				minyan[dow][MARIV].word2 != 
						day[loop].mariv.word2) {
				strcpy(shaharit,format_time(0,
						&(day[loop].shaharit),
						TIME_FORMAT_ACTUAL));
				strcpy(mincha,format_time(0,
						&(day[loop].mincha),
						TIME_FORMAT_ACTUAL));
				strcpy(mariv,format_time(0,
						&(day[loop].mariv),
						TIME_FORMAT_ACTUAL));
				fprintf(file,
	"%-10s %-10s%2d %s   	%s   	%s\n",first ? "Minyan" : "",
			month[current_month],loop,
			shaharit,mincha,mariv);
				first = NO;
			}
		}
	}
}

#if defined(PROTOTYPES)
void print_out(FILE *print_file,FILE *schedule_file) {
#else
void print_out(print_file,schedule_file) FILE *print_file;FILE *schedule_file; {
#endif
	int        loop;
	info_t      *iptr;
	int        item;
	int        item_count;
	int        mark;
	char      **ptr;
	char	   string[256];
	char	  *sptr;
	day_data_t  *tday;

	if (!print_file && !(print_file = fopen(print_file_name,"w"))) {
		sprintf(string,"Can not open file '%s' for the calendar file\n",
				print_file_name);
		report_error(string);
		return;
	}
	if (!schedule_file && !(schedule_file= fopen(schedule_file_name,"w"))) {
		sprintf(string,"Can not open file '%s' for the calendar file\n",
				schedule_file_name);
		report_error(string);
		return;
	}

	for (ptr = ps_front;*ptr;ptr++)
		fprintf(print_file,"%s\n",*ptr);

	fprintf(print_file,"/START_DAY  %d   def\n",first_week_day + 1);
	fprintf(print_file,"/MONTH      (%s - ",month[current_month]);
	for (mark = 0,loop = 1;loop <= numdays;loop++) {
		if ((!mark && day[loop].jmonth) ||
				strcmp(day[mark].jmonth,day[loop].jmonth)!= 0) {
			if (mark)
				fputc('/',print_file);
			fprintf(print_file,"%s",day[loop].jmonth);
			mark = loop;
		}
	}
	fprintf(print_file,") def\n");
	fprintf(print_file,"/MDAYS          %d  def\n",numdays);
	fprintf(print_file,"/YEAR          (%d) def\n",current_year);
	fprintf(print_file,"/BOTTOM_MARGIN  %d  def\n",margin_bottom);
	fprintf(print_file,"/LEFT_MARGIN    %d  def\n",margin_left);
	fprintf(print_file,"/RIGHT_MARGIN   %d  def\n",margin_right);
	fprintf(print_file,"/TOP_MARGIN     %d  def\n",margin_top);
	if (day[1].jyear != day[numdays].jyear)
		fprintf(print_file,"/JYEAR      (%d/%d) def\n",day[1].jyear,
				day[numdays].jyear);
	else
		fprintf(print_file,"/JYEAR      (%d) def\n",day[1].jyear);

	for (ptr = ps_middle;*ptr;ptr++)
		fprintf(print_file,"%s\n",*ptr);

	fprintf(print_file,"(%s) LOCALE\n",city->name);
	for (loop = 1;loop <= numdays;loop++) {
		tday = day + loop;
		item = 0;

		if (tday->jmonth && *tday->jmonth)
			fprintf(print_file,"%2d (%s %d) JDATE\n",loop,
					tday->jmonth,tday->jday);
		if (tday->shaharit.word1)
			fprintf(print_file,"%2d (%s) SHAHARIT\n",loop,
					format_time(loop,&tday->shaharit,TIME_FORMAT_CLIPPED));
		if (tday->mincha.word1)
			fprintf(print_file,"%2d (%s) MINCHA\n",loop,
					format_time(loop,&tday->mincha,TIME_FORMAT_CLIPPED));
		if (tday->mariv.word1)
			fprintf(print_file,"%2d (%s) MARIV\n",loop,
					format_time(loop,&tday->mariv,TIME_FORMAT_CLIPPED));
		if (tday->mitzvah) {
			fprintf(print_file,"%2d %d (%s) (%s) MITZVAH\n",loop,
					tday->mitzvah->place,tday->mitzvah->who,
					mitztypes[tday->mitzvah->type]);
			item++;
			if (tday->mitzvah->place == 1)
				fprintf(schedule_file,
						"%s %2d\t\t%s Mitzvah of %s",
						month[current_month],loop,
						mitztypes[tday->mitzvah->type],
						tday->mitzvah->who);
			else
				fprintf(schedule_file,"%s %2d\t\t%s %s Mitzvah",
						month[current_month],loop,
						tday->mitzvah->who,
						tday->mitzvah->type);
			if (tday->mitzvah->schedule && *tday->mitzvah->schedule)
				fprintf(schedule_file," - %s",
						tday->mitzvah->schedule);
			fputc('\n',schedule_file);
		}
		if (day[loop + 1].mitzvah)
			item++;
		if (tday->holiday || tday->shabbat)
			fprintf(print_file,"%2d HOLIDAY\n",loop);
		if (tday->candles.word1) {
			fprintf(print_file,"%2d (%s) %s\n",loop,
			format_time(loop,&tday->candles,TIME_FORMAT_CLIPPED),
			tday->candles.word1 == TIME_CANDLES_HAVDALAH ? 
					"Havdalah":"Candles");
		}
		if (tday->shabbat) {
			fprintf(print_file,"%2d (%s) (%s) (%s) SHABBAT\n",loop,
					tday->shabbat->name,
					tday->shabbat->torah,
					tday->shabbat->prophet);
			fprintf(schedule_file,"%s %2d\t\tShabbat %s",
					month[current_month],loop,
					tday->shabbat->name);
			if (tday->shabbat->schedule && *tday->shabbat->schedule)
				fprintf(schedule_file," - %s",
					tday->shabbat->schedule);
			fputc('\n',schedule_file);
		}
		if (tday->chanukah)
			fprintf(print_file,"%2d %d CHANUKAH\n",loop,
					tday->chanukah);
		if (tday->omer)
			fprintf(print_file,"%2d %d OMER\n",loop,tday->omer);
		for (iptr = tday->info;iptr;iptr = iptr->next) {
			item_count += break_up(loop,&iptr->time,iptr->calendar,
				      iptr->bold,string);
                        for (sptr = string;*sptr;sptr++)
			  {
			  fprintf(print_file,"%2d (",loop);
			  for (;*sptr;sptr++) {
				  if (*sptr == '(' || *sptr == ')')
					  fputc('\\',print_file);
				  fputc(*sptr,print_file);
			  }
			  fprintf(print_file,") %d %s\n",++item,
					  iptr->bold ? "BOLD_ITEM" : "ITEM");
			  }

			if ((iptr->calendar && *iptr->calendar) ||
					(iptr->schedule && *iptr->schedule)) {
				fprintf(schedule_file,"%s %2d\t",
						month[current_month],loop);
				if (iptr->time.word1)
					fprintf(schedule_file,"%s",
					format_time(loop,&iptr->time,TIME_FORMAT_SHORT));
				fputc('\t',schedule_file);
				if (iptr->calendar && *iptr->calendar) {
					fprintf(schedule_file,"%s",
							iptr->calendar);
					if (iptr->schedule && *iptr->schedule)
						fprintf(schedule_file," - ");
				}
				if (iptr->schedule && *iptr->schedule)
					fprintf(schedule_file,"%s",
							iptr->schedule);
				fputc('\n',schedule_file);
			}
		}
	}
	fprintf(schedule_file,"Candle Lighting times are for %s\n",city->name);

	for (ptr = ps_back;*ptr;ptr++)
		fprintf(print_file,"%s\n",*ptr);

	fclose(print_file);
	fclose(schedule_file);
	return;
}
