static char version[] =
"hcal version 1.0 - postscript hebrew calendar generator\n\n\
copyright 1994,1995 - Steven Weintraub";

/*
 
    hcal version 1.0 - postscript hebrew calendar generator
    copyright 1994,1995 - Steven Weintraub
 
    This program is created and distribed by Steven Weintraub. It
    can be freely distributed without permission as long as this
    head is retained in all files it appears and the LICENCE file
    accompanies it.
 
*/

#include "hcal.h"

#ifndef BATCH
#if defined(PROTOTYPES)
int cur_verify(cur_mlist_t *mitem) {
#else
int cur_verify(mitem) cur_mlist_t *mitem; {
#endif
	city_t		*cptr;
	char		*endptr;
	info_t		*iptr;
	int		 ivalue;
	int		 items;
	int		 maxitems;
	char		 message[MAX_STR];
	char		*sptr;
	time_info_t	 time;

	switch (mitem->type_special) {
		case 'C': /* verify city is unique */
			for (cptr = cities;cptr && cptr->name;cptr = cptr->next) {
				if (casecmp(cptr->name,mitem->s_answer) == 0) {
					cur_msg_win(
	"That city is already in the list");
					return 0;
				}
			}
			break;
		case 'b': /* verify bar mitzvah date */
		case 'S': /* verify Shabbat date */
			ivalue = strtol(mitem->s_answer,&endptr,10);
			if (*endptr) {
				sprintf(message,
					"Day must the integer day in the month");
				cur_msg_win(message);
				return 0;
			}
			mitem->i_answer = ivalue;
			if (ivalue <= 0 || ivalue > numdays) {
				sprintf(message,
	"Day must be a day between 1 and %d",numdays);
				cur_msg_win(message);
				return 0;
			}
			if (day_of_week(current_month,ivalue,current_year)!=6) {
				cur_msg_win("Day must be a Shabbat");
				return 0;
			}
			if (mitem->type_special == 'b' && 
					day[ivalue].mitzvah != NULL) {
				cur_msg_win(
	"You already have a Bar Mitzvah on that day");
				return 0;
			}
			if (mitem->type_special == 'S' && 
					day[ivalue].shabbat != NULL) {
				cur_msg_win(
	"You already have Shabbat information on that day");
				return 0;
			}
			break;
		case 'l': /* validate latitute */
		case 'L': /* validate longitute */
			if (!is_latlong(mitem->s_answer,mitem->type_special,
					NO)) {
				cur_msg_win("Latitude/Longitude in bad format {deg[:min:[sec]] [N|S|E|W]}");
				return 0;
			}
			break;
		case 'd': /* validate day in month */
		case 'D': /* validate day in month for item placement */
			ivalue = strtol(mitem->s_answer,&endptr,10);
			if (*endptr) {
				sprintf(message,
					"Day must the integer day in the month");
				cur_msg_win(message);
				return 0;
			}
			mitem->i_answer = ivalue;
			if (ivalue <= 0 || ivalue > numdays) {
				sprintf(message,
	"Day must be a day between 1 and %d",numdays);
				cur_msg_win(message);
				return 0;
			}
			if (mitem->type_special == 'd')
				break;
			for (iptr = day[ivalue].info,items = 0;iptr;
					iptr = iptr->next)
				items += iptr->items;
			maxitems = 5;
			if (DAY_OF_WEEK(ivalue) == 6)
				maxitems--;
			if ((first_week_day == 6 && numdays == 31) ||
					(first_week_day >= 5 && numdays >= 30))
				maxitems--;
			if (items >= maxitems) {
				cur_msg_win("This date is already full");
				return 0;
			}
			sprintf(message,
	"Room for %d lines of items avaiable on this date",maxitems - items);
			cur_msg_win(message);
			break;
		case 't': /* verify time field */
			if (is_time(mitem->s_answer,NO,&time)) {
				mitem->i_answer = time.word1;
				mitem->i_lower = time.word2;
			} else {
				get_time(mitem);
			}
			if ((mitem->i_answer & TIME_TYPE_MASK)== TIME_TYPE_NONE)
				mitem->i_answer = 0;
			break;
	}
	return 1;
}
#endif

#if defined(PROTOTYPES)
int is_int(char *string) {
#else
int is_int(string) char *string; {
#endif
	if (!string || !*string)
		return NO;
	if (*string == '-' || *string == '+')
		string++;
	while (*string && isdigit(*string))
		string++;
	if (*string)
		return NO;
	return YES;
}

#if defined(PROTOTYPES)
int is_jmonth(char *string) {
#else
int is_jmonth(string) char *string; {
#endif
	if (casecmp(string,"Tishrei") == 0)
		return 0;
	else if (casecmp(string,"Cheshvan") == 0)
		return 1;
	else if (casecmp(string,"Kislev") == 0)
		return 2;
	else if (casecmp(string,"Tevet") == 0)
		return 3;
	else if (casecmp(string,"Sh'vat") == 0)
		return 4;
	else if (casecmp(string,"Adar") == 0)
		return 5;
	else if (casecmp(string,"Adar I") == 0)
		return 5;
	else if (casecmp(string,"Adar II") == 0)
		return 6;
	else if (casecmp(string,"Nisan") == 0)
		return 7;
	else if (casecmp(string,"Iyyar") == 0)
		return 8;
	else if (casecmp(string,"Sivan") == 0)
		return 9;
	else if (casecmp(string,"Tamuz") == 0)
		return 10;
	else if (casecmp(string,"Av") == 0)
		return 11;
	else if (casecmp(string,"Elul") == 0)
		return 12;
	return -1;
}

#if defined(PROTOTYPES)
int is_latlong(char *string,int type,int have_file) {
#else
int is_latlong(string,type,have_file) char *string;int type;int have_file;{
#endif
	int	ch;
	int	degree;
	int	letter;
	int	median;
	int	minute;
	int	retval;
	int	second;
	char   *save_eot;
	int    *save_read_place;
	char   *sptr;
	char   *sptr2;

	degree = 0; 
	minute = 0;
	second = 0;
	save_read_place = read_place;
	save_eot = NULL;
	sptr = string;
	while (isspace(*sptr))
	  sptr++;
	for (degree = 0; isdigit(*sptr);
			degree = (degree * 10) + *sptr - '0',sptr++)
		;
	if (*sptr == ':' || *sptr == ',' || *sptr == ';') {
		*sptr++;
		if (!isdigit(*sptr))
			return NO;
		for (minute = 0; isdigit(*sptr);
				minute = (minute * 10) + *sptr - '0',sptr++)
			;
		if (minute > 59)
			return NO;
		if (*sptr == ':' || *sptr == ',' || *sptr == ';') {
			*sptr++;
			if (!isdigit(*sptr))
				return NO;
			for (second = 0; isdigit(*sptr);second = 
					(second * 10) + *sptr - '0',sptr++)
				;
			if (minute > 59)
				return NO;
		}
	}
	while (*sptr && *sptr == ' ')
		sptr++;
	if (!*sptr && have_file) {
		save_eot = sptr;
		sptr2 = sptr;
		do {
			ch = next_char();
			*sptr2 = ch;
			sptr2++;
		} while (isspace(ch));
		sptr = sptr2 - 1;
		while (ch != EOF && !isspace(ch)) {
			ch = next_char();
			*sptr2 = ch;
			sptr2++;
		}
		replace_char();
		sptr2--;
		*sptr2 = '\0';
	}
	letter = *sptr;
	retval = YES;
	if (type == 'l') {
		if (degree > 89)
			retval = NO;
		if (toupper(letter) != 'N' && toupper(letter) != 'S')
			retval = NO;
	} else {
		if (degree > 179)
			retval = NO;
		if (toupper(letter) != 'E' && toupper(letter) != 'W')
			retval = NO;
	}
	sptr++;
	if (*sptr)
		retval = NO;
	if (!retval) {
		if (save_eot) {
			*save_eot = '\0';
			read_place = save_read_place;
		}
	} else if (type == 'l') {
		sprintf(string,"%2d:%2.2d:%2.2d %c",degree,minute,second,
				toupper(letter));
	} else {
		sprintf(string,"%3d:%2.2d:%2.2d %c",degree,minute,second,
				toupper(letter));
	}
	return retval;
}

#if defined(PROTOTYPES)
int is_month(char *string) {
#else
int is_month(string) char *string; {
#endif
	if (casecmp(string,"January") == 0)
		return 0;
	else if (casecmp(string,"Jan") == 0)
		return 0;
	else if (casecmp(string,"February") == 0)
		return 1;
	else if (casecmp(string,"Feb") == 0)
		return 1;
	else if (casecmp(string,"March") == 0)
		return 2;
	else if (casecmp(string,"Mar") == 0)
		return 2;
	else if (casecmp(string,"April") == 0)
		return 3;
	else if (casecmp(string,"Apr") == 0)
		return 3;
	else if (casecmp(string,"May") == 0)
		return 4;
	else if (casecmp(string,"June") == 0)
		return 5;
	else if (casecmp(string,"Jun") == 0)
		return 5;
	else if (casecmp(string,"July") == 0)
		return 6;
	else if (casecmp(string,"Jul") == 0)
		return 6;
	else if (casecmp(string,"August") == 0)
		return 7;
	else if (casecmp(string,"Aug") == 0)
		return 7;
	else if (casecmp(string,"September") == 0)
		return 8;
	else if (casecmp(string,"Sep") == 0)
		return 8;
	else if (casecmp(string,"Sept") == 0)
		return 8;
	else if (casecmp(string,"October") == 0)
		return 9;
	else if (casecmp(string,"Oct") == 0)
		return 9;
	else if (casecmp(string,"November") == 0)
		return 10;
	else if (casecmp(string,"Nov") == 0)
		return 10;
	else if (casecmp(string,"December") == 0)
		return 11;
	else if (casecmp(string,"Dec") == 0)
		return 11;
	return -1;
}

#if defined(PROTOTYPES)
char *items_fit(time_info_t *time,char *info,int bold,int today) {
#else
char *items_fit(time,info,bold,today) time_info_t *time;char *info;int bold;int today; {
#endif
	int	 items = 0;
	info_t	*iptr;
	int	 maxitems;

	items=break_up(today,time,info,bold,NULL);
	for (iptr = day[today].info;iptr;iptr = iptr->next)
		items += iptr->items;
	maxitems = 5;
	if (DAY_OF_WEEK(today) == 6)
		maxitems--;
	if ((first_week_day == 6 && numdays == 31) ||
		(first_week_day >= 5 && numdays >= 30))
	maxitems--;
	if (items > maxitems)
		return "This would over fill the date - reduce calendar info";
	return NULL;
}

#if defined(PROTOTYPES)
int is_dot(char *string) {
#else
int is_dot(string) char *string; {
#endif
	char    *sptr;

	if (!string || !*string)
		return NO;
	for (sptr = string;*sptr && *sptr != '.';string++)
		;
	if (*sptr)
		return NO;
	return YES;
}

